"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _Arch:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ArchEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Arch.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    ARCH_UNKNOWN: _Arch.ValueType  # 0
    ARCH_ARM64: _Arch.ValueType  # 1

class Arch(_Arch, metaclass=_ArchEnumTypeWrapper): ...

ARCH_UNKNOWN: Arch.ValueType  # 0
ARCH_ARM64: Arch.ValueType  # 1
global___Arch = Arch

@typing.final
class Command(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MAGIC_FIELD_NUMBER: builtins.int
    EXEC_FIELD_NUMBER: builtins.int
    DLOPEN_FIELD_NUMBER: builtins.int
    DLCLOSE_FIELD_NUMBER: builtins.int
    DLSYM_FIELD_NUMBER: builtins.int
    CALL_FIELD_NUMBER: builtins.int
    PEEK_FIELD_NUMBER: builtins.int
    POKE_FIELD_NUMBER: builtins.int
    LIST_DIR_FIELD_NUMBER: builtins.int
    SHOW_OBJECT_FIELD_NUMBER: builtins.int
    SHOW_CLASS_FIELD_NUMBER: builtins.int
    DUMMY_BLOCK_FIELD_NUMBER: builtins.int
    CLOSE_FIELD_NUMBER: builtins.int
    CLASS_LIST_FIELD_NUMBER: builtins.int
    EXEC_CHUNK_FIELD_NUMBER: builtins.int
    CUSTOM_FIELD_NUMBER: builtins.int
    magic: builtins.int
    @property
    def exec(self) -> global___CmdExec: ...
    @property
    def dlopen(self) -> global___CmdDlopen: ...
    @property
    def dlclose(self) -> global___CmdDlclose: ...
    @property
    def dlsym(self) -> global___CmdDlsym: ...
    @property
    def call(self) -> global___CmdCall: ...
    @property
    def peek(self) -> global___CmdPeek: ...
    @property
    def poke(self) -> global___CmdPoke: ...
    @property
    def list_dir(self) -> global___CmdListDir: ...
    @property
    def show_object(self) -> global___CmdShowObject: ...
    @property
    def show_class(self) -> global___CmdShowClass: ...
    @property
    def dummy_block(self) -> global___CmdDummyBlock: ...
    @property
    def close(self) -> global___CmdClose: ...
    @property
    def class_list(self) -> global___CmdGetClassList: ...
    @property
    def exec_chunk(self) -> global___CmdExecChunk: ...
    @property
    def custom(self) -> global___CmdCustom: ...
    def __init__(
        self,
        *,
        magic: builtins.int = ...,
        exec: global___CmdExec | None = ...,
        dlopen: global___CmdDlopen | None = ...,
        dlclose: global___CmdDlclose | None = ...,
        dlsym: global___CmdDlsym | None = ...,
        call: global___CmdCall | None = ...,
        peek: global___CmdPeek | None = ...,
        poke: global___CmdPoke | None = ...,
        list_dir: global___CmdListDir | None = ...,
        show_object: global___CmdShowObject | None = ...,
        show_class: global___CmdShowClass | None = ...,
        dummy_block: global___CmdDummyBlock | None = ...,
        close: global___CmdClose | None = ...,
        class_list: global___CmdGetClassList | None = ...,
        exec_chunk: global___CmdExecChunk | None = ...,
        custom: global___CmdCustom | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["call", b"call", "class_list", b"class_list", "close", b"close", "custom", b"custom", "dlclose", b"dlclose", "dlopen", b"dlopen", "dlsym", b"dlsym", "dummy_block", b"dummy_block", "exec", b"exec", "exec_chunk", b"exec_chunk", "list_dir", b"list_dir", "peek", b"peek", "poke", b"poke", "show_class", b"show_class", "show_object", b"show_object", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["call", b"call", "class_list", b"class_list", "close", b"close", "custom", b"custom", "dlclose", b"dlclose", "dlopen", b"dlopen", "dlsym", b"dlsym", "dummy_block", b"dummy_block", "exec", b"exec", "exec_chunk", b"exec_chunk", "list_dir", b"list_dir", "magic", b"magic", "peek", b"peek", "poke", b"poke", "show_class", b"show_class", "show_object", b"show_object", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["type", b"type"]) -> typing.Literal["exec", "dlopen", "dlclose", "dlsym", "call", "peek", "poke", "list_dir", "show_object", "show_class", "dummy_block", "close", "class_list", "exec_chunk", "custom"] | None: ...

global___Command = Command

@typing.final
class Response(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EXEC_FIELD_NUMBER: builtins.int
    EXEC_CHUNK_FIELD_NUMBER: builtins.int
    DLOPEN_FIELD_NUMBER: builtins.int
    DLCLOSE_FIELD_NUMBER: builtins.int
    DLSYM_FIELD_NUMBER: builtins.int
    PEEK_FIELD_NUMBER: builtins.int
    POKE_FIELD_NUMBER: builtins.int
    CALL_FIELD_NUMBER: builtins.int
    ERROR_FIELD_NUMBER: builtins.int
    DUMMY_BLOCK_FIELD_NUMBER: builtins.int
    SHOW_OBJECT_FIELD_NUMBER: builtins.int
    CLASS_LIST_FIELD_NUMBER: builtins.int
    SHOW_CLASS_FIELD_NUMBER: builtins.int
    LIST_DIR_FIELD_NUMBER: builtins.int
    CUSTOM_FIELD_NUMBER: builtins.int
    @property
    def exec(self) -> global___ResponseCmdExec: ...
    @property
    def exec_chunk(self) -> global___ResponseCmdExecChunk: ...
    @property
    def dlopen(self) -> global___ResponseDlopen: ...
    @property
    def dlclose(self) -> global___ResponseDlclose: ...
    @property
    def dlsym(self) -> global___ResponseDlsym: ...
    @property
    def peek(self) -> global___ResponsePeek: ...
    @property
    def poke(self) -> global___ResponsePoke: ...
    @property
    def call(self) -> global___ResponseCall: ...
    @property
    def error(self) -> global___ResponseError: ...
    @property
    def dummy_block(self) -> global___ResponseDummyBlock: ...
    @property
    def show_object(self) -> global___ResponseShowObject: ...
    @property
    def class_list(self) -> global___ResponseGetClassList: ...
    @property
    def show_class(self) -> global___ResponseShowClass: ...
    @property
    def list_dir(self) -> global___ResponseListdir: ...
    @property
    def custom(self) -> global___ResponseCustom: ...
    def __init__(
        self,
        *,
        exec: global___ResponseCmdExec | None = ...,
        exec_chunk: global___ResponseCmdExecChunk | None = ...,
        dlopen: global___ResponseDlopen | None = ...,
        dlclose: global___ResponseDlclose | None = ...,
        dlsym: global___ResponseDlsym | None = ...,
        peek: global___ResponsePeek | None = ...,
        poke: global___ResponsePoke | None = ...,
        call: global___ResponseCall | None = ...,
        error: global___ResponseError | None = ...,
        dummy_block: global___ResponseDummyBlock | None = ...,
        show_object: global___ResponseShowObject | None = ...,
        class_list: global___ResponseGetClassList | None = ...,
        show_class: global___ResponseShowClass | None = ...,
        list_dir: global___ResponseListdir | None = ...,
        custom: global___ResponseCustom | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["call", b"call", "class_list", b"class_list", "custom", b"custom", "dlclose", b"dlclose", "dlopen", b"dlopen", "dlsym", b"dlsym", "dummy_block", b"dummy_block", "error", b"error", "exec", b"exec", "exec_chunk", b"exec_chunk", "list_dir", b"list_dir", "peek", b"peek", "poke", b"poke", "show_class", b"show_class", "show_object", b"show_object", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["call", b"call", "class_list", b"class_list", "custom", b"custom", "dlclose", b"dlclose", "dlopen", b"dlopen", "dlsym", b"dlsym", "dummy_block", b"dummy_block", "error", b"error", "exec", b"exec", "exec_chunk", b"exec_chunk", "list_dir", b"list_dir", "peek", b"peek", "poke", b"poke", "show_class", b"show_class", "show_object", b"show_object", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["type", b"type"]) -> typing.Literal["exec", "exec_chunk", "dlopen", "dlclose", "dlsym", "peek", "poke", "call", "error", "dummy_block", "show_object", "class_list", "show_class", "list_dir", "custom"] | None: ...

global___Response = Response

@typing.final
class ReturnRegistersArm(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    X0_FIELD_NUMBER: builtins.int
    X1_FIELD_NUMBER: builtins.int
    X2_FIELD_NUMBER: builtins.int
    X3_FIELD_NUMBER: builtins.int
    X4_FIELD_NUMBER: builtins.int
    X5_FIELD_NUMBER: builtins.int
    X6_FIELD_NUMBER: builtins.int
    X7_FIELD_NUMBER: builtins.int
    D0_FIELD_NUMBER: builtins.int
    D1_FIELD_NUMBER: builtins.int
    D2_FIELD_NUMBER: builtins.int
    D3_FIELD_NUMBER: builtins.int
    D4_FIELD_NUMBER: builtins.int
    D5_FIELD_NUMBER: builtins.int
    D6_FIELD_NUMBER: builtins.int
    D7_FIELD_NUMBER: builtins.int
    x0: builtins.int
    x1: builtins.int
    x2: builtins.int
    x3: builtins.int
    x4: builtins.int
    x5: builtins.int
    x6: builtins.int
    x7: builtins.int
    d0: builtins.float
    d1: builtins.float
    d2: builtins.float
    d3: builtins.float
    d4: builtins.float
    d5: builtins.float
    d6: builtins.float
    d7: builtins.float
    def __init__(
        self,
        *,
        x0: builtins.int = ...,
        x1: builtins.int = ...,
        x2: builtins.int = ...,
        x3: builtins.int = ...,
        x4: builtins.int = ...,
        x5: builtins.int = ...,
        x6: builtins.int = ...,
        x7: builtins.int = ...,
        d0: builtins.float = ...,
        d1: builtins.float = ...,
        d2: builtins.float = ...,
        d3: builtins.float = ...,
        d4: builtins.float = ...,
        d5: builtins.float = ...,
        d6: builtins.float = ...,
        d7: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["d0", b"d0", "d1", b"d1", "d2", b"d2", "d3", b"d3", "d4", b"d4", "d5", b"d5", "d6", b"d6", "d7", b"d7", "x0", b"x0", "x1", b"x1", "x2", b"x2", "x3", b"x3", "x4", b"x4", "x5", b"x5", "x6", b"x6", "x7", b"x7"]) -> None: ...

global___ReturnRegistersArm = ReturnRegistersArm

@typing.final
class Argument(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    V_INT_FIELD_NUMBER: builtins.int
    V_DOUBLE_FIELD_NUMBER: builtins.int
    V_STR_FIELD_NUMBER: builtins.int
    V_BYTES_FIELD_NUMBER: builtins.int
    v_int: builtins.int
    v_double: builtins.float
    v_str: builtins.str
    v_bytes: builtins.bytes
    def __init__(
        self,
        *,
        v_int: builtins.int = ...,
        v_double: builtins.float = ...,
        v_str: builtins.str = ...,
        v_bytes: builtins.bytes = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["type", b"type", "v_bytes", b"v_bytes", "v_double", b"v_double", "v_int", b"v_int", "v_str", b"v_str"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["type", b"type", "v_bytes", b"v_bytes", "v_double", b"v_double", "v_int", b"v_int", "v_str", b"v_str"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["type", b"type"]) -> typing.Literal["v_int", "v_double", "v_str", "v_bytes"] | None: ...

global___Argument = Argument

@typing.final
class Handshake(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MAGIC_FIELD_NUMBER: builtins.int
    ARCH_FIELD_NUMBER: builtins.int
    SYSNAME_FIELD_NUMBER: builtins.int
    MACHINE_FIELD_NUMBER: builtins.int
    magic: builtins.int
    arch: global___Arch.ValueType
    sysname: builtins.str
    machine: builtins.str
    def __init__(
        self,
        *,
        magic: builtins.int = ...,
        arch: global___Arch.ValueType = ...,
        sysname: builtins.str = ...,
        machine: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["arch", b"arch", "machine", b"machine", "magic", b"magic", "sysname", b"sysname"]) -> None: ...

global___Handshake = Handshake

@typing.final
class ObjcClass(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    address: builtins.int
    name: builtins.str
    def __init__(
        self,
        *,
        address: builtins.int = ...,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["address", b"address", "name", b"name"]) -> None: ...

global___ObjcClass = ObjcClass

@typing.final
class CmdShowObject(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: builtins.int
    address: builtins.int
    def __init__(
        self,
        *,
        address: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["address", b"address"]) -> None: ...

global___CmdShowObject = CmdShowObject

@typing.final
class ResponseShowObject(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DESCRIPTION_FIELD_NUMBER: builtins.int
    description: builtins.str
    def __init__(
        self,
        *,
        description: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["description", b"description"]) -> None: ...

global___ResponseShowObject = ResponseShowObject

@typing.final
class CmdShowClass(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: builtins.int
    address: builtins.int
    def __init__(
        self,
        *,
        address: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["address", b"address"]) -> None: ...

global___CmdShowClass = CmdShowClass

@typing.final
class ResponseShowClass(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DESCRIPTION_FIELD_NUMBER: builtins.int
    description: builtins.str
    def __init__(
        self,
        *,
        description: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["description", b"description"]) -> None: ...

global___ResponseShowClass = ResponseShowClass

@typing.final
class CmdExecChunk(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BUFFER_FIELD_NUMBER: builtins.int
    buffer: builtins.bytes
    def __init__(
        self,
        *,
        buffer: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["buffer", b"buffer"]) -> None: ...

global___CmdExecChunk = CmdExecChunk

@typing.final
class ResponseCmdExecChunk(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BUFFER_FIELD_NUMBER: builtins.int
    EXIT_CODE_FIELD_NUMBER: builtins.int
    buffer: builtins.bytes
    exit_code: builtins.int
    def __init__(
        self,
        *,
        buffer: builtins.bytes = ...,
        exit_code: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["buffer", b"buffer", "exit_code", b"exit_code", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["buffer", b"buffer", "exit_code", b"exit_code", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["type", b"type"]) -> typing.Literal["buffer", "exit_code"] | None: ...

global___ResponseCmdExecChunk = ResponseCmdExecChunk

@typing.final
class CmdDlopen(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FILENAME_FIELD_NUMBER: builtins.int
    MODE_FIELD_NUMBER: builtins.int
    filename: builtins.str
    mode: builtins.int
    def __init__(
        self,
        *,
        filename: builtins.str = ...,
        mode: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["filename", b"filename", "mode", b"mode"]) -> None: ...

global___CmdDlopen = CmdDlopen

@typing.final
class ResponseDlopen(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HANDLE_FIELD_NUMBER: builtins.int
    handle: builtins.int
    def __init__(
        self,
        *,
        handle: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["handle", b"handle"]) -> None: ...

global___ResponseDlopen = ResponseDlopen

@typing.final
class CmdDlclose(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HANDLE_FIELD_NUMBER: builtins.int
    handle: builtins.int
    def __init__(
        self,
        *,
        handle: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["handle", b"handle"]) -> None: ...

global___CmdDlclose = CmdDlclose

@typing.final
class ResponseDlclose(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    RES_FIELD_NUMBER: builtins.int
    res: builtins.int
    def __init__(
        self,
        *,
        res: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["res", b"res"]) -> None: ...

global___ResponseDlclose = ResponseDlclose

@typing.final
class CmdDlsym(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HANDLE_FIELD_NUMBER: builtins.int
    SYMBOL_NAME_FIELD_NUMBER: builtins.int
    handle: builtins.int
    symbol_name: builtins.str
    def __init__(
        self,
        *,
        handle: builtins.int = ...,
        symbol_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["handle", b"handle", "symbol_name", b"symbol_name"]) -> None: ...

global___CmdDlsym = CmdDlsym

@typing.final
class ResponseDlsym(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PTR_FIELD_NUMBER: builtins.int
    ptr: builtins.int
    def __init__(
        self,
        *,
        ptr: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["ptr", b"ptr"]) -> None: ...

global___ResponseDlsym = ResponseDlsym

@typing.final
class CmdExec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BACKGROUND_FIELD_NUMBER: builtins.int
    ARGV_FIELD_NUMBER: builtins.int
    ENVP_FIELD_NUMBER: builtins.int
    background: builtins.bool
    @property
    def argv(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def envp(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        background: builtins.bool = ...,
        argv: collections.abc.Iterable[builtins.str] | None = ...,
        envp: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["argv", b"argv", "background", b"background", "envp", b"envp"]) -> None: ...

global___CmdExec = CmdExec

@typing.final
class ResponseCmdExec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PID_FIELD_NUMBER: builtins.int
    pid: builtins.int
    def __init__(
        self,
        *,
        pid: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["pid", b"pid"]) -> None: ...

global___ResponseCmdExec = ResponseCmdExec

@typing.final
class CmdCall(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: builtins.int
    VA_LIST_INDEX_FIELD_NUMBER: builtins.int
    ARGV_FIELD_NUMBER: builtins.int
    address: builtins.int
    va_list_index: builtins.int
    @property
    def argv(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Argument]: ...
    def __init__(
        self,
        *,
        address: builtins.int = ...,
        va_list_index: builtins.int = ...,
        argv: collections.abc.Iterable[global___Argument] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["address", b"address", "argv", b"argv", "va_list_index", b"va_list_index"]) -> None: ...

global___CmdCall = CmdCall

@typing.final
class ResponseCall(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ARM_REGISTERS_FIELD_NUMBER: builtins.int
    RETURN_VALUE_FIELD_NUMBER: builtins.int
    return_value: builtins.int
    @property
    def arm_registers(self) -> global___ReturnRegistersArm: ...
    def __init__(
        self,
        *,
        arm_registers: global___ReturnRegistersArm | None = ...,
        return_value: builtins.int = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["arm_registers", b"arm_registers", "return_value", b"return_value", "return_values", b"return_values"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["arm_registers", b"arm_registers", "return_value", b"return_value", "return_values", b"return_values"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["return_values", b"return_values"]) -> typing.Literal["arm_registers", "return_value"] | None: ...

global___ResponseCall = ResponseCall

@typing.final
class CmdPeek(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    address: builtins.int
    size: builtins.int
    def __init__(
        self,
        *,
        address: builtins.int = ...,
        size: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["address", b"address", "size", b"size"]) -> None: ...

global___CmdPeek = CmdPeek

@typing.final
class ResponsePeek(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATA_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    def __init__(
        self,
        *,
        data: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["data", b"data"]) -> None: ...

global___ResponsePeek = ResponsePeek

@typing.final
class CmdPoke(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    address: builtins.int
    data: builtins.bytes
    def __init__(
        self,
        *,
        address: builtins.int = ...,
        data: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["address", b"address", "data", b"data"]) -> None: ...

global___CmdPoke = CmdPoke

@typing.final
class ResponsePoke(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ResponsePoke = ResponsePoke

@typing.final
class CmdListDir(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PATH_FIELD_NUMBER: builtins.int
    path: builtins.str
    def __init__(
        self,
        *,
        path: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["path", b"path"]) -> None: ...

global___CmdListDir = CmdListDir

@typing.final
class CmdDummyBlock(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___CmdDummyBlock = CmdDummyBlock

@typing.final
class ResponseDummyBlock(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ADDRESS_FIELD_NUMBER: builtins.int
    SIZE_FIELD_NUMBER: builtins.int
    address: builtins.int
    size: builtins.int
    def __init__(
        self,
        *,
        address: builtins.int = ...,
        size: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["address", b"address", "size", b"size"]) -> None: ...

global___ResponseDummyBlock = ResponseDummyBlock

@typing.final
class CmdGetClassList(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___CmdGetClassList = CmdGetClassList

@typing.final
class ResponseGetClassList(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CLASSES_FIELD_NUMBER: builtins.int
    @property
    def classes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ObjcClass]: ...
    def __init__(
        self,
        *,
        classes: collections.abc.Iterable[global___ObjcClass] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["classes", b"classes"]) -> None: ...

global___ResponseGetClassList = ResponseGetClassList

@typing.final
class ResponseError(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ResponseError = ResponseError

@typing.final
class ResponseListdir(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MAGIC_FIELD_NUMBER: builtins.int
    DIRP_FIELD_NUMBER: builtins.int
    DIR_ENTRIES_FIELD_NUMBER: builtins.int
    magic: builtins.int
    dirp: builtins.int
    @property
    def dir_entries(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___DirEntry]: ...
    def __init__(
        self,
        *,
        magic: builtins.int = ...,
        dirp: builtins.int = ...,
        dir_entries: collections.abc.Iterable[global___DirEntry] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["dir_entries", b"dir_entries", "dirp", b"dirp", "magic", b"magic"]) -> None: ...

global___ResponseListdir = ResponseListdir

@typing.final
class DirEntry(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    D_TYPE_FIELD_NUMBER: builtins.int
    D_NAME_FIELD_NUMBER: builtins.int
    LSTAT_FIELD_NUMBER: builtins.int
    STAT_FIELD_NUMBER: builtins.int
    d_type: builtins.int
    d_name: builtins.str
    @property
    def lstat(self) -> global___DirEntryStat: ...
    @property
    def stat(self) -> global___DirEntryStat: ...
    def __init__(
        self,
        *,
        d_type: builtins.int = ...,
        d_name: builtins.str = ...,
        lstat: global___DirEntryStat | None = ...,
        stat: global___DirEntryStat | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["lstat", b"lstat", "stat", b"stat"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["d_name", b"d_name", "d_type", b"d_type", "lstat", b"lstat", "stat", b"stat"]) -> None: ...

global___DirEntry = DirEntry

@typing.final
class DirEntryStat(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ERRNO1_FIELD_NUMBER: builtins.int
    ST_DEV_FIELD_NUMBER: builtins.int
    ST_MODE_FIELD_NUMBER: builtins.int
    ST_NLINK_FIELD_NUMBER: builtins.int
    ST_INO_FIELD_NUMBER: builtins.int
    ST_UID_FIELD_NUMBER: builtins.int
    ST_GID_FIELD_NUMBER: builtins.int
    ST_RDEV_FIELD_NUMBER: builtins.int
    ST_SIZE_FIELD_NUMBER: builtins.int
    ST_BLOCKS_FIELD_NUMBER: builtins.int
    ST_BLKSIZE_FIELD_NUMBER: builtins.int
    ST_ATIME1_FIELD_NUMBER: builtins.int
    ST_MTIME1_FIELD_NUMBER: builtins.int
    ST_CTIME1_FIELD_NUMBER: builtins.int
    errno1: builtins.int
    st_dev: builtins.int
    st_mode: builtins.int
    st_nlink: builtins.int
    st_ino: builtins.int
    st_uid: builtins.int
    st_gid: builtins.int
    st_rdev: builtins.int
    st_size: builtins.int
    st_blocks: builtins.int
    st_blksize: builtins.int
    st_atime1: builtins.int
    st_mtime1: builtins.int
    st_ctime1: builtins.int
    def __init__(
        self,
        *,
        errno1: builtins.int = ...,
        st_dev: builtins.int = ...,
        st_mode: builtins.int = ...,
        st_nlink: builtins.int = ...,
        st_ino: builtins.int = ...,
        st_uid: builtins.int = ...,
        st_gid: builtins.int = ...,
        st_rdev: builtins.int = ...,
        st_size: builtins.int = ...,
        st_blocks: builtins.int = ...,
        st_blksize: builtins.int = ...,
        st_atime1: builtins.int = ...,
        st_mtime1: builtins.int = ...,
        st_ctime1: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["errno1", b"errno1", "st_atime1", b"st_atime1", "st_blksize", b"st_blksize", "st_blocks", b"st_blocks", "st_ctime1", b"st_ctime1", "st_dev", b"st_dev", "st_gid", b"st_gid", "st_ino", b"st_ino", "st_mode", b"st_mode", "st_mtime1", b"st_mtime1", "st_nlink", b"st_nlink", "st_rdev", b"st_rdev", "st_size", b"st_size", "st_uid", b"st_uid"]) -> None: ...

global___DirEntryStat = DirEntryStat

@typing.final
class CmdClose(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___CmdClose = CmdClose

@typing.final
class CmdCustom(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONTENT_FIELD_NUMBER: builtins.int
    content: builtins.bytes
    def __init__(
        self,
        *,
        content: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["content", b"content"]) -> None: ...

global___CmdCustom = CmdCustom

@typing.final
class ResponseCustom(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONTENT_FIELD_NUMBER: builtins.int
    content: builtins.bytes
    def __init__(
        self,
        *,
        content: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["content", b"content"]) -> None: ...

global___ResponseCustom = ResponseCustom
